/*-------------------------------------------------------------------*/
/* CB2PA.C      Mikko Malinen                                        */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Converts given Codebook and Trainingset to Partition              */
/* or, alternatively,                                                */
/* converts given Partition and Trainingset to Codebook.             */
/*                                                                   */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define ProgName       "CB2PA"
#define VersionNumber  "V.0.13"
#define LastUpdated    "04.08.09" /* MM */

/* ----------------------------------------------------------------- */
/*      Changelog:                                                   */
/*                                                                   */
/* MM     0.13  All in one module                                    */
/* MM     0.12  Console interface separated, cb routines used        */
/* MM     0.11  pa -> cb conversion added, 20.7.09                   */
/* MM     0.10  Original version with cb-> pa conversion, 17.7.09    */
/*                                                                   */
/* ----------------------------------------------------------------- */


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>

#define  FACTFILE  "cb2pa.fac"

#include "parametr.c"
#include "cb.h"
#include "file.h"
#include "memctrl.h"
#include "owntypes.h"
#include "interfc.h"


/* ========================== PROTOTYPES ============================= */

static void PrintOperatingInfo(char  TSetName[],
                               char  CBName[],
                               char  OutPAName[]);
void PrintInfo(void);
static void ShutDown(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* P);
char* InfoLine(void);


/* ========================== FUNCTIONS ============================== */


static void PrintOperatingInfo(char  TSetName[],
                               char  CBName[],
                               char  OutPAName[])
{
  if( Value(QuietLevel) >= 2 )
    {
      printf("\n%s %s %s\n", ProgName, VersionNumber, LastUpdated);
      printf("Training Set: %s\n", TSetName);
      printf("Codebook:     %s\n", CBName);
      printf("Partitioning: %s\n", OutPAName);
      PrintSelectedOptions();
      printf("\n");
    }
  else if( Value(QuietLevel) >= 1 )
    {
      printf("%-12s ", TSetName);
      printf("%-12s ", CBName);
    }
}


/*-------------------------------------------------------------------*/


void PrintInfo(void)
{
  printf("\n%s %s %s\n", ProgName, VersionNumber, LastUpdated);
  printf("This program converts given codebook and trainingset to partition\\
n");
  printf("or, alternatively, it converts given partition and trainingset to codebook.\n");
  printf("Usage (cb->pa): %s [%coption] <training-set> <codebook>\n", ProgName, OPTION_SYMBOL);
  printf("or (pa->cb):    %s [%coption] <training-set> <partitioning>\n", ProgName, OPTION_SYMBOL);

  printf("For example: %s bridge bridge-km\n", ProgName);
  printf("\n  Options:\n");
  PrintOptions();
  printf("\n%s %s %s\n", ProgName, VersionNumber, LastUpdated);
}

/* ================================================================= */


static void ShutDown(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* P)
{
  FreeCodebook(TS);
  FreeCodebook(CB);
  FreePartitioning(P);
}


/* ================================================================= */


char* InfoLine(void)
{
  char* p;
  int len;

  len = strlen(ProgName)+strlen(VersionNumber)+strlen(LastUpdated)+4;
  p   = (char*) malloc(len*sizeof(char));

  if (!p)
    {
      ErrorMessage("ERROR: Allocating memory failed!\n");
      ExitProcessing(FATAL_ERROR);
    }

  sprintf(p, "%s\t%s\t%s", ProgName, VersionNumber, LastUpdated);

  return p;
} 


/*===========================  M A I N  ==============================*/


int main(int argc, char* argv[])
{
  CODEBOOK      CB;
  TRAININGSET   TS;
  PARTITIONING  P;
  char          TSName[80];
  char          CBName[80];
  char          OutPAName[80] = {'\0'};
  char          OutCBName[80] = {'\0'}; 
  CBFILETYPE    ft1, ft2;
  char *        genMethod;

  ParameterInfo paraminfo[2] =  { { TSName, "", 0, INFILE  },
                                  { CBName, "", 0, INFILE  } };

  ParseParameters( argc, argv, 2, paraminfo);
  ft1 = DetermineCBFileTypeConsideringOrder(TSName, 12);
  ft2 = DetermineCBFileTypeConsideringOrder(CBName, 231);

  if ((ft1 == NOTFOUND) || (ft2 == NOTFOUND))
    {
      ErrorMessage("\nERROR: You must give two valid files: one ts file and one cb or pa file.\n\n");
      PrintInfo();
      ExitProcessing(FATAL_ERROR);
    }
  if (ft1 != TSFILE)
    {
      ErrorMessage("\nERROR: The first given file must be a ts file.\n\n");
      PrintInfo();
      ExitProcessing(FATAL_ERROR);
    }
  if ((ft2 != CBFILE) && (ft2 != PAFILE))
    {
      ErrorMessage("\nERROR: You must give a cb or pa file as input file.\n\n");
      PrintInfo();
      ExitProcessing(FATAL_ERROR);
    }

  if (ft2 == CBFILE)   /* cb -> pa conversion */
    {
      PickFileName(CBName,OutPAName);
      CheckFileName(OutPAName, FormatNamePA);
      PrintOperatingInfo(TSName, CBName, OutPAName);
      ReadTrainingSet(TSName, &TS);
      ReadCodebook(CBName, &CB);
      CreateNewPartitioning(&P, &TS, BookSize(&CB)); /* Initialize partit. */
      if (BookSize(&TS)!=P.TSsize)
	{
	  ErrorMessage("\nERROR: Training set and partitioning sizes do not match.\n\n");
	  ShutDown(&TS, &CB, &P);
	  ExitProcessing(FATAL_ERROR);
	}
      if (BookSize(&CB)!=P.PartitionCount)
        {
          ErrorMessage("\nERROR: Codebook size and partition count do not match.\n\n");
          ShutDown(&TS, &CB, &P);
          ExitProcessing(FATAL_ERROR);
        }
      GenerateOptimalPartitioning(&TS, &CB, &P);
      WritePartitioning(OutPAName,&P,&TS,Value(OverWrite));
    }

  else if (ft2 == PAFILE)     /* pa -> cb conversion */ 
    {
      PickFileName(CBName,OutCBName);  /* CBName is in this case PA filename */
      CheckFileName(OutCBName, FormatNameCB);
      PrintOperatingInfo(TSName, OutCBName, CBName);
      ReadTrainingSet(TSName, &TS);
      ReadPartitioning(CBName, &P, &TS);
      CreateNewCodebook(&CB, PartitionCount(&P), &TS); /* Initializes codeb. */
      if (BookSize(&TS)!=P.TSsize)
	{
          ErrorMessage("\nERROR: Training set and partitioning sizes do not match\n\n");
          ShutDown(&TS, &CB, &P);
          ExitProcessing(FATAL_ERROR);
	}
      GenerateOptimalCodebook(&TS, &CB, &P);
      genMethod = InfoLine();
      AddGenerationMethod(&CB, genMethod);
      WriteCodebook(OutCBName,&CB,Value(OverWrite));
      free(genMethod);
    }

  ShutDown(&TS, &CB, &P);
  checkmemory();
  return( 0 );
}
